import urllib2
import json
import re
import time
import logging
from datetime import datetime, timedelta
from StringIO import StringIO
from django.core.management.base import BaseCommand, CommandError

from optparse import make_option

from dashboard.models import (
        Bug,
        Build,
        Result,
        ResultLog,
        Run,
        )
from dashboard.management import (
        jenkins_get,
        BUILD_COUNT_MAX,
        JOB_COUNT_MAX,
        DEBUG_LIMITS,
        )

#l = logging.getLogger('django.db.backends')
#l.setLevel(logging.DEBUG)
#l.addHandler(logging.StreamHandler())

from dashboard.utils import regexes

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
            make_option('-I', '--initial-run',
                action='store_true',
                dest='initial_run',
                default=False,
                help='Initial run processes all data (takes longer)'),
            )
                
    help = 'Pull data from jenkins'

    def get_build_number(self, build_url):
        media_info_url = "{}artifact/media_info".format(build_url)
        try:
            response = jenkins_get(media_info_url, as_json=False)
        except urllib2.HTTPError:
            response = ""

        m = regexes['build-number'].match(response)

        if m:
            return m.group(1)
        else:
            return None

    def handle(self, *args, **options):
        starttime = datetime.now()
        initial_run = options.get('initial_run')

        # for the initial run include older releases as well.
        if initial_run:
            regexes['smoke'] = re.compile(ur"^(lucid|natty|oneiric|precise|quantal)-(desktop|server|alternate)")

        jobs_top = jenkins_get('https://jenkins.qa.ubuntu.com/api/json')
        jobs = jobs_top['jobs']


        job_count = 0 # XXX: DEBUG
        for job in jobs:
            name = job['name']
            url  = job['url']
            self.stdout.write('Name: %s\n' % name)
            if regexes['bootspeed'].match(name):
                self.stdout.write("Bootspeed test %s\n" % name)
            elif regexes['upgrade'].match(name):
                self.stdout.write("Upgrade test %s\n" % name)
            elif regexes['smoke'].match(name):
                self.stdout.write("Smoke test %s\n" % name)
                flavor = 'ubuntu'
                if regexes['smoke-ec2'].match(name):
                    if regexes['smoke-ec2-daily'].match(name):
                        m = regexes['smoke-ec2-daily-data'].match(name)
                        release, variant = m.group(1,2)
                else:
                    m = regexes['smoke-data'].match(name)
                    orig_name = name
                    release, variant, arch, name = m.group(1,2,3,4)

                    # XXX: debugging limit
                    job_count += 1
                    if DEBUG_LIMITS and job_count > JOB_COUNT_MAX:
                        break

                    self.stdout.write("Release: %s, variant: %s, arch: %s, name: %s\n" % (release, variant, arch, name))

                    # Ignore static validation per Gema's request via IRC.
                    if name == "static_validation":
                        self.stdout.write("Skipping static validation test: {}".format(orig_name))
                        continue
                    

                    # XXX: See if adding '?depth=1' makes this faster.

                    job_info = jenkins_get("%s/api/json/?depth=1" % url)
                    builds = job_info['builds']
                    last_build = datetime.fromtimestamp(job_info['lastBuild']['timestamp']/1000)

                    # for update runs only check jobs that have built in the
                    # last week.
                    if not initial_run and datetime.now() - last_build > timedelta(days=7):
                        continue

                    build_count = 0 # XXX: DEBUG
                    for build in builds:
                        # XXX: debugging limit
                        build_count += 1
                        if DEBUG_LIMITS and build_count > BUILD_COUNT_MAX:
                            break

                        build_number = build['number']
                        build_url    = build['url']
                        #build_info   = jenkins_get("%s/api/json" % build_url)
                        building     = build['building']
                        build_desc   = build['description']
                        build_date   = datetime.fromtimestamp(build['timestamp']/1000)
                        artifacts    = build['artifacts']
                        actions      = build['actions']

                        # Skip this Jenkins build if it hasn't finished yet
                        if building:
                            continue

                        # XXX: find out what TZ jenkins uses and use correct one here
                        if not initial_run and datetime.now() - build_date > timedelta(days=7):
                            continue

                        build_no = self.get_build_number(build_url)

                        lp_bugs  = []

                        if build_desc != None:
                            b = build_desc.split(',')
                            b = regexes['build_desc'].findall(build_desc)
                            for word in b:
                                word = word.strip()
                                if regexes['buildid'].match(word) and build_no is None:
                                    # It's a build number
                                    build_no = "{} ?".format(word)
                                elif regexes['lpbug'].match(word):
                                    lp_bugs.append(word[4:])

                        # if all else fails set to the build_date
                        if build_no is None:
                            build_no = build_date.strftime("%Y%m%d ?")

                        self.stdout.write("Build #%s, date: %s, build_no: %s\n" % (build_number, build_date, build_no))

                        run = None
                        try:
                            run = Run.objects.get(release=release, flavor=flavor, build_no=build_no)
                        except Run.DoesNotExist:
                            run = Run.objects.create(release=release, flavor=flavor, build_no=build_no, ran_at=build_date, test_type=0)

                        build = None
                        try:
                            build = run.build_set.get(variant=variant, arch=arch, flavor=flavor)
                        except Build.DoesNotExist:
                            build = run.build_set.create(variant=variant, arch=arch, flavor=flavor)

                        result = None
                        try:
                            result = build.result_set.get(name=name, jenkins_build=build_number)
                            # break
                        except Result.DoesNotExist:
                            fail_count, skip_count, pass_count, total_count = (0, 0, 0, 0)
                            for action in actions:
                                try:
                                    fail_count, skip_count, total_count = (action['failCount'], action['skipCount'], action['totalCount'])
                                    pass_count = total_count - fail_count - skip_count
                                    break
                                except KeyError:
                                    continue

                            result = build.result_set.create(name=name, jenkins_build=build_number, ran_at=build_date, jenkins_url=build_url, fail_count=fail_count, skip_count=skip_count, pass_count=pass_count, total_count=total_count)

                        for bug_no in lp_bugs:
                            bug = None
                            try:
                                bug = Bug.objects.get(bug_no = bug_no)
                            except Bug.DoesNotExist:
                                bug = Bug.objects.create(bug_no=bug_no, status='unknown')
                            if bug not in list(result.bugs.all()):
                                result.bugs.add(bug)
                                result.save()

                        for artifact in artifacts:
                            path = artifact['relativePath']

                            # skip artifacts we don't care about.
                            if regexes['artifact-ignore'].match(path):
                                continue

                            remote_url = "%s/artifact/%s" % (build_url, path)
                            art_args = {
                                    'display_name': path,
                                    'remote_url': remote_url,
                                    'path': path,
                                    }
                            try:
                                res_log = result.resultlog_set.get(**art_args)
                            except ResultLog.DoesNotExist:
                                result.resultlog_set.create(**art_args)

            elif regexes['kernel_sru'].match(name):
                self.stdout.write("Kernel SRU test %s\n" %name)


        endtime = datetime.now()

        self.stdout.write("time: {}\n".format(endtime - starttime))
