import re
from datetime import datetime, timedelta
from optparse import make_option
from django.core.management.base import BaseCommand, CommandError

from dashboard.management import (
        jenkins_get,
        BUILD_COUNT_MAX,
        JOB_COUNT_MAX,
        DEBUG_LIMITS,
        )
from dashboard.utils import regexes
from dashboard.models import (
        Kernel,
        SRUResult,
        SRUResultLog,
        )
def _get_kernel_version(description, default="Unknown"):
    """
    Parse a jenkins build description for a kernel version
    """
    version = default

    if description is None:
        description = ''

    r = regexes['kernel_sru-version']

    m = r.match(description)

    if m:
        version = m.group(1)

    return version

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
            make_option('-I', '--initial-run',
                action='store_true',
                dest='initial_run',
                default=False,
                help='Initial run processes all data (takes longer)'),
            )
                
    help = 'Pull data from jenkins'

    def handle(self, *args, **options):
        starttime = datetime.now()
        initial_run = options.get('initial_run')

        url = "https://jenkins.qa.ubuntu.com/api/json"
        jobs_top = jenkins_get(url)

        jobs = jobs_top['jobs']

        job_count = 0 # XXX: DEBUG
        for job in jobs:
            name = job['name']
            m = regexes['kernel_sru'].match(name)
            if m:
                pre_job_type = m.group(1)
                post_job_type = m.group(2)
                job_type = "{}{}".format(pre_job_type, post_job_type)

                pre_release = m.group(3)
                post_release = m.group(4)
                release = "{}{}".format(pre_release, post_release)

                variant, arch, gpu = m.group(5, 6, 7)


                # XXX: debugging limit
                job_count += 1
                if DEBUG_LIMITS and job_count > JOB_COUNT_MAX:
                    break
                
                self.stdout.write("job: {}\n".format(name))
                self.stdout.write("\tjob_type: {}\n".format(job_type))
                self.stdout.write("\trelease: {}\n".format(release))
                self.stdout.write("\tkernel: {}\n".format(variant))
                self.stdout.write("\tarch: {}\n".format(arch))
                self.stdout.write("\tgpu: {}\n".format(gpu))

                url = job['url']
                job_info = jenkins_get("{}/api/json/?depth=1".format(url))
                builds = job_info['builds']
                last_build = datetime.fromtimestamp(job_info['lastBuild']['timestamp']/1000)
                # for update runs only check jobs that have built in the
                # last week.
                if not initial_run and datetime.now() - last_build > timedelta(days=7):
                    continue

                build_count = 0 # XXX: DEBUG
                for build in builds:
                    # XXX: debugging limit
                    build_count += 1
                    if DEBUG_LIMITS and build_count > BUILD_COUNT_MAX:
                        break

                    build_number = build['number']
                    build_url    = build['url']
                    #build_info   = jenkins_get("%s/api/json" % build_url)
                    building     = build['building']
                    build_desc   = build['description']
                    build_date   = datetime.fromtimestamp(build['timestamp']/1000)
                    artifacts    = build['artifacts']
                    actions      = build['actions']

                    if build_desc is None:
                        build_desc = ''

                    kernel_version = _get_kernel_version(build_desc,
                            "Unknown. Ran on {}".format(build_date.strftime("%Y-%m-%d %H:%M")))

                    # Skip this Jenkins build if it hasn't finished yet
                    if building:
                        continue

                    # XXX: find out what TZ jenkins uses and use correct one here
                    if not initial_run and datetime.now() - build_date > timedelta(days=7):
                        continue

                    lp_bugs  = []

                    if build_desc != None:
                        b = build_desc.split(',')
                        b = regexes['build_desc'].findall(build_desc)
                        for word in b:
                            word = word.strip()
                            if regexes['buildid'].match(word) and build_no is None:
                                # It's a build number
                                build_no = "{} ?".format(word)
                            elif regexes['lpbug'].match(word):
                                lp_bugs.append(word[4:])

                    try:
                        kernel = Kernel.objects.get(release=release,
                                                    variant=variant,
                                                    arch=arch,
                                                    gpu=gpu,
                                                    version=kernel_version,
                                                    job_type=job_type)
                    except Kernel.DoesNotExist:
                        kernel = Kernel.objects.create(release=release,
                                                       variant=variant,
                                                       arch=arch,
                                                       gpu=gpu,
                                                       version=kernel_version,
                                                       job_type=job_type)

                    result = None
                    try:
                        result = kernel.sruresult_set.get(name=name, jenkins_build=build_number)
                        # break
                    except SRUResult.DoesNotExist:
                        fail_count, skip_count, pass_count, total_count = (0, 0, 0, 0)
                        for action in actions:
                            try:
                                fail_count, skip_count, total_count = (action['failCount'], action['skipCount'], action['totalCount'])
                                pass_count = total_count - fail_count - skip_count
                                break
                            except KeyError:
                                continue

                        result = kernel.sruresult_set.create(name=name, jenkins_build=build_number, ran_at=build_date, jenkins_url=build_url, fail_count=fail_count, skip_count=skip_count, pass_count=pass_count, total_count=total_count)
