from django.db import models
import datetime

class DashboardBaseModel(models.Model):
    """
    Common base class that provides fields needed on every model.
    """
    class Meta:
        abstract = True

    created_at    = models.DateTimeField(auto_now_add = True)
    updated_at    = models.DateTimeField(auto_now = True)
    internal      = models.BooleanField(default=True)
    publish       = models.BooleanField(default=True)

class Bug(DashboardBaseModel):
    class Meta:
        db_table = 'bugs'
    bug_no        = models.CharField(max_length=255)
    status        = models.CharField(max_length=255)

    def __unicode__(self):
        return self.bug_no

    @property
    def debug_str(self):
        return "{}-{}-{}-{}-{}".format(
                self.id,
                self.bug_no,
                self.status,
                self.created_at,
                self.updated_at,
                )

###
# Smoke
###
class ResultLog(DashboardBaseModel):
    class Meta:
        db_table = 'result_logs'
    display_name  = models.CharField(max_length=255, null = True)
    path          = models.TextField() 
    result        = models.ForeignKey('Result')
    remote_url    = models.URLField()

    def __unicode__(self):
        return self.path


class Run(DashboardBaseModel):
    class Meta:
        db_table = 'runs'

    build_no   = models.CharField(max_length=200, db_index = True)
    release    = models.CharField(max_length=200, db_index = True)
    flavor     = models.CharField(max_length=200, db_index = True)
    test_type  = models.IntegerField(null = True)
    ran_at     = models.DateTimeField('date run')

    # Used by MySerializer to add property fields to the JSON representation
    # of an instance of this class.
    extra_fields = [
            'pass_count',
            'fail_count',
            'skip_count',
            'total_count',
            'pass_rate',
            'pass_rate_pct',
            'bug_count',
            ]

    @property
    def pass_count(self):
        passes = 0
        for build in self.build_set.all():
            passes += build.pass_count
        return passes

    @property
    def fail_count(self):
        fails = 0
        for build in self.build_set.all():
            fails += build.fail_count
        return fails

    @property
    def skip_count(self):
        skips = 0
        for build in self.build_set.all():
            skips += build.skip_count
        return skips

    @property
    def total_count(self):
        total = 0
        for build in self.build_set.all():
            total += build.total_count
        return total

    @property
    def pass_rate(self):
        total = 0.0
        count = 0
        for build in self.build_set.all():
            total += build.pass_rate
            count += 1

        return total / count if count > 0 else 0

    @property
    def pass_rate_pct(self):
        return "%d%%" % int(self.pass_rate*100)

    @property
    def bug_count(self):
        bugs = 0
        for build in self.build_set.all():
            bugs = bugs + build.bug_count
        return bugs

class Build(DashboardBaseModel):
    class Meta:
        db_table = 'builds'
    run        = models.ForeignKey(Run)
    name       = models.CharField(max_length=200, null = True)
    flavor     = models.CharField(max_length=200)
    variant    = models.CharField(max_length=200)
    arch       = models.CharField(max_length=200)

    # Used by MySerializer to add property fields to the JSON representation
    # of an instance of this class.
    extra_fields = [
            'pass_count',
            'fail_count',
            'skip_count',
            'total_count',
            'pass_rate',
            'pass_rate_pct',
            'bug_count',
            ]

    @property
    def debug_str(self):
        return "{}-{}-{}-{}-{}-{}".format(
                self.id,
                self.name,
                self.flavor,
                self.arch,
                self.created_at,
                self.updated_at,
                )

    @property
    def pass_count(self):
        return self.result_set.aggregate(models.Sum('pass_count'))['pass_count__sum'] or 0

    @property
    def fail_count(self):
        return self.result_set.aggregate(models.Sum('fail_count'))['fail_count__sum'] or 0

    @property
    def skip_count(self):
        return self.result_set.aggregate(models.Sum('skip_count'))['skip_count__sum'] or 0

    @property
    def total_count(self):
        return self.result_set.aggregate(models.Sum('total_count'))['total_count__sum'] or 0

    @property
    def pass_rate(self):
        """
        total = 0.0
        count = 0
        for result in self.result_set.all():
            total += result.pass_rate
            count += 1
        return total / count if count > 0 else 0
        """
        return self.pass_count*1.0 / self.total_count if self.total_count > 0 else 0

    @property
    def pass_rate_pct(self):
        return "%d%%" % int(self.pass_rate*100)

    @property
    def bug_count(self):
        return self.result_set.annotate(num_bugs=models.Count('bugs')).aggregate(models.Sum('num_bugs'))['num_bugs__sum'] or 0


class Result(DashboardBaseModel):
    class Meta:
        db_table = 'results'
    build         = models.ForeignKey(Build)
    bugs          = models.ManyToManyField(Bug)
    # django doesn't have proper polymorphic associations, so mis-use a
    # many-to-many relationship for the logs, even if each log can only
    # belong to one other object.
    #result_logs   = models.ManyToManyField(ResultLog)
    name          = models.CharField(max_length=255)
    jenkins_build = models.CharField(max_length=255)
    jenkins_url   = models.URLField()
    fail_count    = models.IntegerField()
    skip_count    = models.IntegerField()
    pass_count    = models.IntegerField()
    total_count   = models.IntegerField()
    ran_at        = models.DateTimeField('date run')

    @property
    def debug_str(self):
        return "{}-{}-{}-{}-{}".format(
                self.id,
                self.name,
                self.build,
                self.created_at,
                self.updated_at,
                )

    # Used by MySerializer to add property fields to the JSON representation
    # of an instance of this class.
    extra_fields = [
            'pass_rate',
            'pass_rate_pct',
            'bug_count',
            'log_count',
            ]

    @property
    def pass_rate(self):
        return self.pass_count*1.0 / self.total_count if self.total_count > 0 else 0

    @property
    def pass_rate_pct(self):
        return "%d%%" % int(self.pass_rate*100)

    @property
    def bug_count(self):
        return self.bugs.count()

    @property
    def log_count(self):
        return self.resultlog_set.count()

###
# Kernel SRU
###

class Kernel(DashboardBaseModel):
    class Meta:
        db_table = "kernels"

    job_type = models.CharField(max_length=100)
    release = models.CharField(max_length=100)
    variant = models.CharField(max_length=100)
    arch = models.CharField(max_length=100)
    gpu = models.CharField(max_length=100)
    version = models.CharField(max_length=100)

    # Used by MySerializer to add property fields to the JSON representation
    # of an instance of this class.
    extra_fields = [
            'pass_count',
            'skip_count',
            'fail_count',
            'total_count',
            'pass_rate',
            'pass_rate_pct',
            'bug_count',
            'log_count',
            ]

    def debug_str(self):
        return "{}-{}-{}-{}".format(
                self.release,
                self.variant,
                self.arch,
                self.gpu,
                )

    @property
    def name(self):
        return "{}-{}".format(self.release, self.variant)

    @property
    def total_count(self):
        return self.sruresult_set.aggregate(models.Sum('total_count'))['total_count__sum'] or 0

    @property
    def pass_count(self):
        return self.sruresult_set.aggregate(models.Sum('pass_count'))['pass_count__sum'] or 0

    @property
    def fail_count(self):
        return self.sruresult_set.aggregate(models.Sum('fail_count'))['fail_count__sum'] or 0

    @property
    def skip_count(self):
        return self.sruresult_set.aggregate(models.Sum('skip_count'))['skip_count__sum'] or 0

    @property
    def pass_rate(self):
        return self.pass_count*1.0 / self.total_count if self.total_count > 0 else 0

    @property
    def pass_rate_pct(self):
        return "%d%%" % int(self.pass_rate*100)

    @property
    def bug_count(self):
        return self.sruresult_set.annotate(num_bugs=models.Count('bugs')).aggregate(models.Sum('num_bugs'))['num_bugs__sum'] or 0

    @property
    def log_count(self):
        return self.sruresultlog_set.count()

    @property
    def last_run_date(self):
        return self.sruresult_set.aggregate(models.Max('ran_at'))['ran_at__max'].strftime("%Y-%m-%d %H:%M")

# XXX: refactor this and Result
class SRUResult(DashboardBaseModel):
    class Meta:
        db_table = 'sru_results'

    kernel = models.ForeignKey(Kernel)
    bugs = models.ManyToManyField(Bug)

    jenkins_build = models.CharField(max_length=255)
    jenkins_url = models.URLField()
    name = models.CharField(max_length=255)
    pass_count = models.IntegerField()
    fail_count = models.IntegerField()
    skip_count = models.IntegerField()
    total_count = models.IntegerField()
    ran_at = models.DateTimeField('date run')

    # Used by MySerializer to add property fields to the JSON representation
    # of an instance of this class.
    extra_fields = [
            'pass_rate',
            'pass_rate_pct',
            'bug_count',
            'log_count',
            ]

    @property
    def pass_rate(self):
        return self.pass_count*1.0 / self.total_count if self.total_count > 0 else 0

    @property
    def pass_rate_pct(self):
        return "%d%%" % int(self.pass_rate*100)

    @property
    def bug_count(self):
        return self.bugs.count()

    @property
    def log_count(self):
        return self.resultlog_set.count()

class SRUResultLog(DashboardBaseModel):
    class Meta:
        db_table = 'sru_result_logs'
    display_name = models.CharField(max_length=255, null = True)
    path = models.TextField() 
    sru_result = models.ForeignKey('SRUResult')
    remote_url = models.URLField()

    def __unicode__(self):
        return self.path

# Bootspeed Models
class Image(DashboardBaseModel):
    class Meta:
        db_table = 'images'
        unique_together = ('release', 'variant', 'arch', 'build_number')

    release = models.CharField(max_length=200)
    variant = models.CharField(max_length=200)
    arch = models.CharField(max_length=200)
    md5 = models.CharField(max_length=200, unique=True)
    build_number = models.CharField(max_length=50)

    def __unicode__(self):
        return "{}-{}-{}-{}".format(
                self.release,
                self.variant,
                self.arch,
                self.build_number,
                )
    # Used by MySerializer to add property fields to the JSON representation
    # of an instance of this class.
    extra_fields = [
            'min_boot',
            'max_boot',
            'avg_boot',
            'result_count',
            ]

    def get_chart_data(self, start=None, end=None):
        x_data = []
        avg_data = []
        max_data = []
        min_data = []
        low_error = []
        hi_error = []
        end_date = end if end is not None else datetime.datetime.now()
        start_date = start if start is not None else end_date - datetime.timedelta(days=60)
        results = self.imageresult_set.filter(ran_at__gte=start_date, ran_at__lte=end_date).values('ran_at').annotate(avg_boot=models.Avg('boot'), max_boot=models.Max('boot'), min_boot=models.Min('boot')).order_by('ran_at')
        for result in results:
            x_data.append(result['ran_at'])
            avg_data.append(result['avg_boot'])
            max_data.append(result['max_boot'])
            min_data.append(result['min_boot'])
            low_error.append(result['avg_boot'] - result['min_boot'])
            hi_error.append(result['max_boot'] - result['avg_boot'])

        return x_data, avg_data, max_data, min_data, low_error, hi_error

    @property
    def result_count(self):
        return self.imageresult_set.count()

    @property
    def avg_boot(self):
        return self.imageresult_set.aggregate(models.Avg('boot'))['boot__avg']

    @property
    def min_boot(self):
        return self.imageresult_set.aggregate(models.Min('boot'))['boot__min']

    @property
    def max_boot(self):
        return self.imageresult_set.aggregate(models.Max('boot'))['boot__max']

    @property
    def last_run_date(self):
        return self.imageresult_set.aggregate(models.Max('ran_at'))['ran_at__max'].strftime("%Y-%m-%d %H:%M")

class Machine(DashboardBaseModel):
    class Meta:
        db_table = 'machines'
    name = models.CharField(max_length=200, unique=True)
    mac_address = models.CharField(max_length=200)

    # Used by MySerializer to add property fields to the JSON representation
    # of an instance of this class.
    extra_fields = [
            'result_count',
            ]

    @property
    def result_count(self):
        return ImageResult.objects.filter(machine=self).count()

class ImageResult(DashboardBaseModel):
    class Meta:
        db_table = 'images_results'

    image = models.ForeignKey(Image)
    machine = models.ForeignKey(Machine)
    ran_at = models.DateTimeField('date run')

    kernel_init = models.FloatField()
    kernel = models.FloatField()
    plumbing = models.FloatField()
    xorg = models.FloatField()
    desktop = models.FloatField()
    boot = models.FloatField()

