import json

from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import (
        require_GET,
        require_POST,
        )

from django.shortcuts import (
        get_object_or_404,
        )

from dashboard.utils import (
        JSONResponse,
        MySerializer,
        require_API_TOKEN,
        )

from dashboard.models import (
        Image,
        ImageResult,
        Machine,
        )

from dashboard.forms import (
        ImageResultAddForm,
        )

MAX_RESULTS=180

@require_GET
def overall(request, arch):
    # do some reversed sorting to get the latest images.
    images = reversed(Image.objects.filter(arch=arch).order_by('-build_number')[:MAX_RESULTS])

    images_data = []
    for image in images:
        images_data.append({
            'min_boot': image.min_boot,
            'max_boot': image.max_boot,
            'avg_boot': image.avg_boot,
            'bottom': image.min_boot,
            'middle': image.avg_boot - image.min_boot,
            'top': image.max_boot - image.avg_boot,
            'build_number': image.build_number,
            })

    data = json.dumps(images_data)

    return JSONResponse(data)

@require_GET
def machine_list(request):
    machines = Machine.objects.all()

    serializer = MySerializer(use_extra_fields=True)
    data = serializer.serialize(machines)

    return JSONResponse(data)


@require_GET
def machine_detail(request, machine_id):
    machine = get_object_or_404(Machine, id=machine_id)

    serializer = MySerializer(use_extra_fields=True)
    data = serializer.serialize([machine])

    return JSONResponse(data)

@require_GET
def image_list(request):
    images = Image.objects.all()

    serializer = MySerializer(use_extra_fields=True)
    data = serializer.serialize(images)

    return JSONResponse(data)

@require_GET
def image_detail(request, image_id):
    image = get_object_or_404(Image, id=image_id)

    serializer = MySerializer(use_extra_fields=True)
    data = serializer.serialize([image])

    return JSONResponse(data)

@require_GET
def image_result_list(request, image_id):
    results = ImageResult.objects.filter(image=image_id)

    serializer = MySerializer(use_extra_fields=True)
    data = serializer.serialize(results)

    return JSONResponse(data)


@require_GET
def arch_list(request):
    """
    List images filtered by arch (i.e. i386, amd64)
    """
    image_arches = Image.objects.values('arch').distinct()

    arches = []
    for arch in image_arches:
        arches.append(arch['arch'])

    data = json.dumps({'arches': arches})

    return JSONResponse(data)


@require_GET
def result_count(request):
    error_data = {
            'result': 'error',
            'message': 'failure',
            }

    machine_name = request.GET.get('machine')
    machine_id = request.GET.get('machine_id')
    md5 = request.GET.get('md5')

    if (machine_id is None and machine_name is None) or md5 is None:
        error_data['message'] = "both machine and md5 are required"
        return JSONResponse(json.dumps(error_data))

    machine_token = None
    if machine_id is not None:
        machine_token = machine_id
        machines = Machine.objects.filter(id=machine_id)
    else:
        machine_token = machine_name
        machines = Machine.objects.filter(name=machine_name)

    if machines.count() != 1:
        error_data['message'] = "unknown machine: {}".format(machine_token)
        return JSONResponse(json.dumps(error_data))

    machine = machines[0]
    image = Image.objects.filter(md5=md5)

    if image.count() != 1:
        error_data['message'] = "unknown md5: {}".format(md5)
        return JSONResponse(json.dumps(error_data))

    image_results = image.imageresult_set.filter(machine=machine)

    data = json.dumps({
        "image_id": image.id,
        "machine_id": machine.id,
        "result_count": image_results.count(),
        })

    return JSONResponse(data)

@require_POST
@csrf_exempt
@require_API_TOKEN
def add_result(request):
    """
    Add a bootspeed run result to the database.

    If the machine can't be found a new one is created.
    If the image can't be found a new one is created.
    An ImageResult isn't added if one with the same data already exists.
    """
    form = ImageResultAddForm(request.POST)
    ret_data = {'result': 'failure'}

    if form.is_valid():
        form_data = form.cleaned_data
        ret_data = []

        machine, created = Machine.objects.get_or_create(name=form_data['machine'])
        ret_data.append(machine)

        image, created = Image.objects.get_or_create(
                release=form_data['release'],
                variant=form_data['variant'],
                arch=form_data['arch'],
                md5=form_data['md5'],
                build_number=form_data['build_number'],
                )
        ret_data.append(image)

        kernel=form_data['kernel']
        kernel_init=form_data['kernel_init']
        plumbing=form_data['plumbing']
        xorg=form_data['xorg']
        desktop=form_data['desktop']
        boot = kernel + kernel_init + plumbing + xorg + desktop

        image_result, created = ImageResult.objects.get_or_create(
                image=image,
                machine=machine,
                kernel=kernel,
                kernel_init=kernel_init,
                plumbing=plumbing,
                xorg=xorg,
                desktop=desktop,
                boot=boot,
                ran_at=form_data['ran_at'],
                )

        ret_data.append(image_result)

        serializer = MySerializer(use_extra_fields=True)
        data = serializer.serialize(ret_data)
    else:
        ret_data['message'] = form.errors
        data = json.dumps(ret_data)

    return JSONResponse(data)
