import json
from django.http import HttpResponse, Http404
from django.views.decorators.csrf import csrf_exempt

from django.views.decorators.http import (
        require_GET,
        require_POST,
        )

from dashboard.forms import (
        ResultSmokeAddForm,
        ResultKernelSRUAddForm,
        BugForm,
        )

from dashboard.models import (
        Bug,
        Build,
        Run,
        Result,
        ResultLog,
        )

from dashboard.utils import (
        get_smoke_data,
        JSONResponse,
        require_API_TOKEN,
        MySerializer,
        get_verbose,
        )

@require_GET
def api_runs(request, release=None):
    runs = Run.objects.all().exclude(publish=False)

    if release is not None:
        runs = runs.filter(release=release)

    use_extra_fields = get_verbose(request)
    serializer = MySerializer(use_extra_fields=get_verbose(request))
    data = serializer.serialize(runs)
    return HttpResponse(data, content_type='application/json')


@require_GET
def api_run(request, run_id):
    # For some magical reason, Django won't serialize if you use .get(). It
    # really needs a queryset, hence the .filter()
    run = Run.objects.filter(id=run_id).exclude(publish=False)
    serializer = MySerializer(use_extra_fields=get_verbose(request, True))
    data = serializer.serialize(run)
    return HttpResponse(data, content_type='application/json')


@require_GET
def api_images(request, run_id):
    try:
        run = Run.objects.exclude(publish=False).get(id=run_id)
    except Run.DoesNotExist:
        raise Http404
    images = run.build_set.all().exclude(publish=False)
    serializer = MySerializer(use_extra_fields=get_verbose(request))
    data = serializer.serialize(images)
    return HttpResponse(data, content_type='application/json')


@require_GET
def api_image(request, run_id, image_id):
    # For some magical reason, Django won't serialize if you use .get(). It
    # really needs a queryset, hence the .filter()
    image = Build.objects.filter(id=image_id).exclude(publish=False)
    serializer = MySerializer(use_extra_fields=get_verbose(request, True))
    data = serializer.serialize(image)
    return HttpResponse(data, content_type='application/json')


@require_GET
def api_results(request, run_id, image_id):
    try:
        image = Build.objects.exclude(publish=False).get(id=image_id)
    except Build.DoesNotExist:
        raise Http404
    results = image.result_set.all().exclude(publish=False)
    serializer = MySerializer(use_extra_fields=get_verbose(request))
    data = serializer.serialize(results)
    return HttpResponse(data, content_type='application/json')


@require_GET
def api_result(request, run_id, image_id, result_id):
    # For some magical reason, Django won't serialize if you use .get(). It
    # really needs a queryset, hence the .filter()
    result = Result.objects.filter(id=result_id).exclude(publish=False)
    serializer = MySerializer(use_extra_fields=get_verbose(request, True))
    data = serializer.serialize(result)
    return HttpResponse(data, content_type='application/json')


@require_GET
def api_stats(request, release=None):
    """
    API view to show counts of model instances.
    """
    runs = Run.objects.all().exclude(publish=False)

    if release is not None:
        runs = runs.filter(release=release).exclude(publish=False)

    stats = {
            'Runs': runs.count(),
            'Builds': runs.annotate(build_count=models.Count('build')).aggregate(models.Sum('build_count'))['build_count__sum'] or 0,
            'Results': runs.annotate(result_count=models.Count('build__result')).aggregate(models.Sum('result_count'))['result_count__sum'] or 0,
            'Bugs': runs.annotate(bug_count=models.Count('build__result__bugs')).aggregate(models.Sum('bug_count'))['bug_count__sum'] or 0,
            'Logs': runs.annotate(log_count=models.Count('build__result__resultlog')).aggregate(models.Sum('log_count'))['log_count__sum'] or 0
            }

    return HttpResponse(json.dumps(stats), content_type='application/json')

@require_POST
@csrf_exempt
@require_API_TOKEN
def api_add_result(request):
    """
    API view to add a new Result.  Will create a Run, Build, and Result
    if they do not exist.
    """
    form = ResultSmokeAddForm(request.POST)

    data = {'result': 'failure'}

    if form.is_valid():
        name = form.cleaned_data['build_name']

        smoke_data = get_smoke_data(name)
        if smoke_data is not None:
            data['result'] = 'success'
            data['smoke_name'] = name
            data['release'] = smoke_data['release']
            data['variant'] = smoke_data['variant']
            data['arch'] = smoke_data['arch']
            data['result_name'] = smoke_data['name']

            build_no = form.cleaned_data['build_no']
            ran_at = form.cleaned_data['ran_at']
            flavor = form.cleaned_data['flavor']

            fail_count = form.cleaned_data['fail_count']
            skip_count = form.cleaned_data['skip_count']
            pass_count = form.cleaned_data['pass_count']

            release = smoke_data['release']
            variant = smoke_data['variant']
            arch = smoke_data['arch']
            testsuite_name = smoke_data['name']

            run = Run.objects.get_or_create(
                    build_no=build_no,
                    ran_at=ran_at,
                    release=release,
                    flavor=flavor,
                    )[0]

            build = Build.objects.get_or_create(
                    run=run,
                    flavor=flavor,
                    variant=variant,
                    arch=arch,
                    )[0]

            result = Result.objects.get_or_create(
                    build=build,
                    name=testsuite_name,
                    jenkins_build='non-jenkins',
                    jenkins_url='non-jenkins',
                    fail_count=fail_count,
                    skip_count=skip_count,
                    pass_count=pass_count,
                    total_count=fail_count + pass_count + skip_count,
                    ran_at=ran_at,
                    )[0]
                    
            data = [result]

            serializer = MySerializer(use_extra_fields=get_verbose(request, True))
            data = serializer.serialize(data)
        else:
            data['message'] = "'{}' did not match the smoke test pattern".format(name)
    else:
        data['message'] = form.errors

    if type(data) != str:
        data = json.dumps(data)


    return JSONResponse(data)


@require_POST
@csrf_exempt
@require_API_TOKEN
def api_result_add_bug(request, run_id, image_id, result_id):
    """
    API view to add a bug to an existing result.
    """
    ret_data = {'result': 'failure'}

    form = BugForm(request.POST)

    try:
        result = Result.objects.exclude(publish=False).get(id=result_id)
    except Result.DoesNotExist:
        ret_data['message'] = 'Result not found'
        data = json.dumps([ret_data])
        return JSONResponse(data)

    if form.is_valid():
        bug_no = form.cleaned_data['bug_no']
        status = form.cleaned_data['status']

        # Only look for a bug with the bug ID
        # XXX: Once we support both community and canonical submissions
        # we will need to determine if we should allow updating the bug
        # status via the API, and if so, who should have access to which
        # bugs.
        bug, created = result.bugs.get_or_create(bug_no=bug_no)
        if created:
            bug.status = status
            bug.save()

        serializer = MySerializer(use_extra_fields=get_verbose(request, True))
        data = serializer.serialize([bug])
    else:
        ret_data['errors'] = form.errors
        data = json.dumps([ret_data])

    return JSONResponse(data)
