import json

from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import (
        require_GET,
        require_POST,
        )
from dashboard.forms import (
        ResultKernelSRUAddForm,
        BugForm,
        )
from dashboard.models import (
        Kernel,
        SRUResult,
        )

from dashboard.utils import (
        JSONResponse,
        MySerializer,
        require_API_TOKEN,
        get_sru_data,
        get_verbose,
        )

@require_POST
@csrf_exempt
@require_API_TOKEN
def api_add_result(request):
    """
    API view to add a new Result.  Will create a Run, Build, and Result
    if they do not exist.

    Returns: the result object on success or an error dictionary on failure.
    """
    form = ResultKernelSRUAddForm(request.POST)

    data = {'result': 'failure'}

    if form.is_valid():
        name = form.cleaned_data['build_name']

        sru_data = get_sru_data(name)
        if sru_data is not None:
            data['result'] = 'success'
            data['sru_name'] = name
            data['release'] = sru_data['release']
            data['variant'] = sru_data['variant']
            data['arch'] = sru_data['arch']
            data['gpu'] = sru_data['gpu']

            data['result_name'] = name

            build_no = form.cleaned_data['build_no']
            ran_at = form.cleaned_data['ran_at']
            flavor = form.cleaned_data['flavor']
            job_type = form.cleaned_data['job_type']
            version = form.cleaned_data['version']

            fail_count = form.cleaned_data['fail_count']
            skip_count = form.cleaned_data['skip_count']
            pass_count = form.cleaned_data['pass_count']

            release = sru_data['release']
            variant = sru_data['variant']
            arch = sru_data['arch']
            gpu = sru_data['gpu']

            kernel = Kernel.objects.get_or_create(
                    job_type=job_type,
                    release=release,
                    variant=variant,
                    arch=arch,
                    gpu=gpu,
                    version=version,
                    )[0]

            result = SRUResult.objects.get_or_create(
                    kernel=kernel,
                    name=name,
                    jenkins_build='non-jenkins',
                    jenkins_url='non-jenkins',
                    fail_count=fail_count,
                    skip_count=skip_count,
                    pass_count=pass_count,
                    total_count=fail_count + pass_count + skip_count,
                    ran_at=ran_at,
                    )[0]
                    
            data = [result]

            serializer = MySerializer(use_extra_fields=get_verbose(request, True))
            data = serializer.serialize(data)
        else:
            data['message'] = "'{}' did not match the sru test pattern".format(name)
    else:
        data['message'] = form.errors

    if type(data) != str:
        data = json.dumps(data)


    return JSONResponse(data)
