import datetime

from django.db import models
from django.views.decorators.http import (
        require_GET,
        require_POST,
        )

from django.http import HttpResponse
from django.shortcuts import (
        render_to_response,
        get_object_or_404,
        )

import matplotlib
matplotlib.use('Agg')
from matplotlib.backends.backend_agg import FigureCanvasAgg
from matplotlib.dates import date2num
from pylab import figure, plot, gca, plot_date, bar

from dashboard.models import (
        Image,
        Machine,
        ImageResult,
        )

def _get_bootspeed_releases():
    releases = []
    for image in Image.objects.values('release').distinct().exclude(publish=False).order_by('-release'):
        releases.append(image['release'])

    return releases

@require_GET
def overview(request, release=None):
    images = Image.objects.all()
    releases = _get_bootspeed_releases()

    if release is not None:
        images = images.filter(release=release)

    data = {
            'images': images,
            'releases': releases,
            }

    return render_to_response('bootspeed/overview.html', data)

@require_GET
def image_overview(request, image_id):
    image = get_object_or_404(Image, id=image_id)

    data = {
            'image': image,
            'results': image.imageresult_set.values('ran_at').annotate(
                kernel_init_avg=models.Avg('kernel_init'),
                kernel_init_min=models.Min('kernel_init'),
                kernel_init_max=models.Max('kernel_init'),

                kernel_avg=models.Avg('kernel'),
                kernel_min=models.Min('kernel'),
                kernel_max=models.Max('kernel'),

                plumbing_avg=models.Avg('plumbing'),
                plumbing_min=models.Min('plumbing'),
                plumbing_max=models.Max('plumbing'),

                xorg_avg=models.Avg('xorg'),
                xorg_min=models.Min('xorg'),
                xorg_max=models.Max('xorg'),

                desktop_avg=models.Avg('desktop'),
                desktop_min=models.Min('desktop'),
                desktop_max=models.Max('desktop'),

                boot_avg=models.Avg('boot'),
                boot_min=models.Min('boot'),
                boot_max=models.Max('boot'),
                ),
            }

    return render_to_response('bootspeed/image_overview.html', data)

CHART_WIDTH_DAYS = 60
@require_GET
def build_boot_chart(request, image_id):
    image = Image.objects.get(id=image_id)

    end = datetime.datetime.now()
    start = end - datetime.timedelta(days=CHART_WIDTH_DAYS)
    
    x_data, avg_data, max_data, min_data, low_error, hi_error = image.get_chart_data(
            start=start,
            end=end,
            )

    f = figure(figsize=(12,9)) # inches
    f.patch.set_alpha(0.0) # transparent background


    #plot_date(date2num(x_data), avg_data, 'b-')
    #plot_date(date2num(x_data), max_data, 'g-')
    #plot_date(date2num(x_data), min_data, 'r-')
    ax = gca()
    ax.bar(x_data, avg_data, color='#dd4814', yerr=[low_error, hi_error], ecolor='b')
    ax.set_title('Bootspeed (sec.)')
    labels = ax.get_xticklabels()
    for label in labels:
        label.set_rotation(90)
        label.set_size('small')
    ax.set_xlim(date2num(start), date2num(end))
    ax.xaxis.set_major_locator(
            matplotlib.dates.DayLocator()
            )
    ax.xaxis.set_major_formatter(
            matplotlib.dates.DateFormatter('%Y%m%d')
            )
    response = HttpResponse(content_type='image/png')
    canvas = FigureCanvasAgg(f)
    canvas.print_png(response, transparent=True)
    matplotlib.pyplot.close(f)

    return response

@require_GET
def arch_overview(request, arch):
    return render_to_response('bootspeed/arch_overview.html', {'arch': arch})
