import matplotlib
import json

from django.core import serializers
from django.http import HttpResponse, Http404
from django.shortcuts import render_to_response
from django.template.loader import get_template
from django.template import Context
from django.views.decorators.http import (
        require_GET,
        require_POST,
        )
from django.views.decorators.csrf import csrf_exempt

from dashboard.models import (
        Build,
        Result,
        Run,
        )

# XXX: This is needed due to a yet to be found import ordering bug.
matplotlib.use('Agg')
from matplotlib.backends.backend_agg import FigureCanvasAgg
from pylab import figure, axes, pie, title


from dashboard.forms import (
        ResultSmokeAddForm,
        ResultKernelSRUAddForm,
        BugForm,
        )
from dashboard.utils import (
        get_smoke_data,
        require_API_TOKEN,
        JSONResponse,
        MySerializer,
        )

from qa_dashboard.settings import API_TOKEN

def _get_releases():
    releases = []
    for run in Run.objects.values('release').order_by('-release').distinct().exclude(publish=False):
        releases.append(run['release'])

    return releases

@require_GET
def overview(request, release, is_flat=False):
    data = {
            'release': release,
            'releases': _get_releases(),
            'is_flat': False,
            }

    if request.GET.get('flat') is not None:
        is_flat = True

    if is_flat:
        data['is_flat'] = True

    # Eager loading of all the associations down to the bug linking table
    # would increase performance significantly. Seems though as if django
    # doesn't support eager loading any further than at most one level
    # of association.
    if is_flat:
        data['items'] = Build.objects.filter(
                run__release=release).exclude(publish=False)
    else:
        data['items'] = Run.objects.filter(
                release=release).exclude(publish=False)

    return render_to_response('smoke/overview.html', data)


@require_GET
def build_overview(request, release, run_id):
    try:
        run = Run.objects.exclude(publish=False).get(id=run_id)
    except Run.DoesNotExist:
        raise Http404

    # Again, eager loading of associations two levels deeper would be great.
    builds = run.build_set.all().exclude(publish=False)
    return render_to_response('smoke/build_overview.html', locals())


@require_GET
def results(request, release, run_id, image_id):
    try:
        image = Build.objects.exclude(publish=False).get(id=image_id)
    except Build.DoesNotExist:
        raise Http404

    results = image.result_set.all().exclude(publish=False)
    return render_to_response('smoke/results.html', locals())


@require_GET
def results_logs(request, release, run_id, image_id, result_id):
    try:
        image  = Build.objects.exclude(publish=False).get(id=image_id)
        result = Result.objects.exclude(publish=False).get(id=result_id)
    except Result.DoesNotExist, Build.DoesNotExist:
        raise Http404

    logs   = result.resultlog_set.all().exclude(publish=False)

    return render_to_response('smoke/result_logs.html', locals())


def _make_pie_chart(obj):
    f = figure(figsize=(6,6)) # in inches... (dpi=80)
    f.patch.set_alpha(0.0) # Transparent background

    pre_data = [
            ('Passed', obj.pass_count, 0.1, '#abffab'),
            ('Failed', obj.fail_count, 0.1, '#ffabab'),
            ('Skipped', obj.skip_count, 0.1, '#ffffa0'),
            ]

    data = []
    labels = []
    explode = []
    colors = []
    for label, value, expl, color in pre_data:
        if value > 0.0:
            data.append(value)
            labels.append(label)
            explode.append(expl)
            colors.append(color)

    pie(data, explode=explode, labels=labels, colors=colors, autopct='%1.1f%%', shadow=True)
    response = HttpResponse(content_type='image/png')
    canvas = FigureCanvasAgg(f)    
    canvas.print_png(response, transparent=True)
    matplotlib.pyplot.close(f)
    return response

@require_GET
def build_pie(request, release, run_id):
    try:
        run = Run.objects.exclude(publish=False).get(id=run_id)
    except Run.DoesNotExist:
        raise Http404

    return _make_pie_chart(run)

@require_GET
def results_pie(request, release, run_id, image_id):
    try:
        image = Build.objects.exclude(publish=False).get(id=image_id)
    except Build.DoesNotExist:
        raise Http404

    return _make_pie_chart(image)
