from django.shortcuts import render_to_response
from django.views.decorators.http import require_GET
from django.db.models import Sum, F

from dashboard.models import Kernel
from dashboard.utils import (
        JSONResponse,
        MySerializer,
        get_verbose,
        )

def _get_kernel_names():
    kernels = []
    for kernel in  Kernel.objects.values('variant').distinct().exclude(publish=False).order_by('variant'):
        kernels.append(kernel['variant'])

    return kernels

def _get_kernel_releases():
    releases = []
    for kernel in Kernel.objects.values('release').distinct().exclude(publish=False).order_by('-release'):
        releases.append(kernel['release'])

    return releases

@require_GET
def api_overview(request, release='', version=''):
    kernels = Kernel.objects.all()

    if release != '':
        kernels = kernels.filter(release=release)

    if version != '':
        kernels = kernels.filter(version=version)

    serializer = MySerializer(use_extra_fields=get_verbose(request, True))
    data = serializer.serialize(kernels)

    return JSONResponse(data)

@require_GET
def overview(request, release=''):
    data = {}
    data['kernel_names'] = _get_kernel_names()
    data['releases'] = _get_kernel_releases()
    data['kernels'] = []
    data['show_unknown'] = True

    show_unknown = request.GET.get('show_unknown')
    print("DEBUG: JAT: show_unknown: {}".format(show_unknown))

    kernels = Kernel.objects.all()

    if show_unknown is None or show_unknown != "1":
        data['show_unknown'] = False
        kernels = kernels.exclude(version__startswith="Unknown")

    if release != '':
        kernels = kernels.filter(release=release)

    kernel_data = {}
    for kernel in kernels:
        version = kernel.version
        if version not in kernel_data:
            kernel_data[version] = {
                    'version': version,
                    'pass_count': 0,
                    'fail_count': 0,
                    'skip_count': 0,
                    'total_count': 0,
                    'bug_count': 0,
                    }

        kernel_data[version]['pass_count'] += kernel.pass_count
        kernel_data[version]['skip_count'] += kernel.skip_count
        kernel_data[version]['fail_count'] += kernel.fail_count
        kernel_data[version]['total_count'] += kernel.total_count
        kernel_data[version]['bug_count'] += kernel.bug_count
        kernel_data[version]['release'] = kernel.release
        kernel_data[version]['last_run_date'] = kernel.last_run_date


    for k,v in kernel_data.iteritems():
        if v['total_count'] == 0:
            v['pass_rate'] = 0
        else:
            v['pass_rate'] = v['pass_count'] * 1.0 / v['total_count']

        v['pass_rate_pct'] = "{:<.2%}".format(v['pass_rate'])

        data['kernels'].append(v)

    return render_to_response('sru/overview.html', data)

@require_GET
def overview_kernel(request, release=None, kernel_version=None):
    kernels = Kernel.objects.filter(version=kernel_version)

    data = {'kernels': kernels}
    data['releases'] = _get_kernel_releases()

    totals = {
            'skip_count': 0,
            'pass_count': 0,
            'fail_count': 0,
            'total_count': 0,
            'pass_rate': 0.0,
            }

    for k in kernels:
        totals['skip_count'] += k.skip_count
        totals['pass_count'] += k.pass_count
        totals['fail_count'] += k.fail_count
        totals['total_count'] += k.total_count

    if totals['total_count'] == 0:
        totals['pass_rate'] = 0.0
    else:
        totals['pass_rate'] = totals['pass_count'] * 1.0 / totals['total_count']
    totals['pass_rate_pct'] = "{:.2%}".format(totals['pass_rate'])

    data['totals'] = totals

    return render_to_response('sru/overview_kernel.html', data)
