"""
Tests for the bootspeed API.

*** Requirements ***
- There must be at least one Image object in the database
- There must be at least one Machine object in the database
- The devserver must be running on localhost:8000 or update
  'base_url'.
"""

import json
import unittest
import urllib2

class TestBootspeedAPI(unittest.TestCase):
    """
    Bootspeed API tests.
    """
    base_url = "http://localhost:8000/api/bootspeed/"

    def _get_json_data(self, url):
        """
        Send a request to 'url' and try to decode the resonse as JSON and
        return the result.
        """
        f = urllib2.urlopen(url)
        response = f.read()
        
        data = json.loads(response)

        return data

    def _get_object_list(self, object_type="image"):
        """
        Retrieve a list of objects from the API.
        """
        url = "{}{}/".format(self.base_url, object_type)

        return self._get_json_data(url)

    def _get_object_details(self, object_id, object_type="image"):
        """
        Retrieve the detail view of an object from the API.
        """
        url = "{}{}/{}".format(self.base_url, object_type, object_id)
        return self._get_json_data(url)

    def test_image_list(self):
        """
        Test that the image list works.
        """
        data = self._get_object_list(object_type="image")
        self.assertTrue(len(data) > 0)

    def test_image_details(self):
        """
        Test that the image details works.
        """
        data = self._get_object_list(object_type="image")
        self.assertTrue(len(data) > 0)

        image_id = data[0]['id']

        data = self._get_object_details(image_id, object_type="image")

        self.assertEqual(len(data), 1)

    def test_invalid_image_details(self):
        """
        Test that an invalid image id results in a 404

        XXX: Perhaps this should return a JSON error message.
        """
        image_id = 0
        with self.assertRaises(urllib2.HTTPError):
            self._get_object_details(image_id, object_type="image")

    def test_machine_list(self):
        """
        Test that the machine list works.
        """
        data = self._get_object_list(object_type="machine")
        self.assertTrue(len(data) > 0)

    def test_machine_details(self):
        """
        Test that the machine details works.
        """
        data = self._get_object_list(object_type="machine")
        self.assertTrue(len(data) > 0)

        machine_id = data[0]['id']

        data = self._get_object_details(machine_id, object_type="machine")

        self.assertEqual(len(data), 1)

    def test_invalid_machine_details(self):
        """
        Test that an invalid machine id results in a 404

        XXX: Perhaps this should return a JSON error message.
        """
        machine_id = 0
        with self.assertRaises(urllib2.HTTPError):
            self._get_object_details(machine_id, object_type="machine")

    def test_image_result_list(self):
        """
        Test that image/result/ returns list of results.
        """

        data = self._get_object_list(object_type="image")
        self.assertTrue(len(data) > 0)

        image_id = data[0]['id']

        data = self._get_json_data("{}image/{}/result/".format(
            self.base_url,
            image_id,
            ))

        self.assertTrue(len(data) > 0)
